@@ -266,6 +266,11 @@ sub LimitCustomField {
         EXPRESSION => 'main.id'
     );
 
+    $self->Limit(
+        LEFTJOIN => $ObjectValuesAlias,
+        FIELD    => 'ObjectType',
+        VALUE    => 'RT::Article',
+    );
     $self->Limit(
         LEFTJOIN => $ObjectValuesAlias,
         FIELD    => 'Disabled',
@@ -970,6 +975,11 @@ sub SimpleSearch {
     return $self;
 }
 
+sub CurrentUserCanSeeAll {
+    my $self = shift;
+    return $self->CurrentUser->HasRight( Right => 'ShowArticle', Object => RT->System ) ? 1 : 0;
+}
+
 RT::Base->_ImportOverlays();
 
 1;
