@@ -389,20 +389,28 @@ sub SetSubValues {
 
 }
 
+=head2 Object
+
+Returns the object current attribute blongs to.
+
+CAVEAT: the returned object is cached, reload it to get the latest data.
+
+=cut
 
 sub Object {
     my $self = shift;
-    my $object_type = $self->__Value('ObjectType');
-    my $object;
-    eval { $object = $object_type->new($self->CurrentUser) };
-    unless(UNIVERSAL::isa($object, $object_type)) {
-        $RT::Logger->error("Attribute ".$self->Id." has a bogus object type - $object_type (".$@.")");
-        return(undef);
-     }
-    $object->Load($self->__Value('ObjectId'));
-
-    return($object);
-
+    unless ( $self->{_cached}{Object} ) {
+        my $object_type = $self->__Value('ObjectType');
+        my $object;
+        eval { $object = $object_type->new( $self->CurrentUser ) };
+        unless ( UNIVERSAL::isa( $object, $object_type ) ) {
+            $RT::Logger->error( "Attribute " . $self->Id . " has a bogus object type - $object_type (" . $@ . ")" );
+            return (undef);
+        }
+        $object->Load( $self->__Value('ObjectId') );
+        $self->{_cached}{Object} = $object;
+    }
+    return $self->{_cached}{Object};
 }
 
 
@@ -1031,6 +1039,22 @@ sub PostInflateFixup {
     }
     elsif ($self->Name eq 'Subscription') {
         my $content = $self->Content;
+        for my $type ( qw/Users Groups/ ) {
+            if ( my $list = $content->{Recipients}{$type} ) {
+                my @ids;
+                for my $item ( @$list ) {
+                    if ( ref $item eq 'SCALAR' ) {
+                        my $obj = $importer->LookupObj($$item);
+                        push @ids, $obj->Id if $obj && $obj->Id;
+                    }
+                    else {
+                        push @ids, $item;
+                    }
+                }
+                @$list = @ids;
+            }
+        }
+
         if (ref($content->{DashboardId}) eq 'SCALAR') {
             my $attr = $importer->LookupObj(${ $content->{DashboardId} });
             if ($attr) {
@@ -1046,6 +1070,17 @@ sub PostInflateFixup {
         }
         $self->SetContent( $content, SyncLinks => 0, RecordTransaction => 0 );
     }
+    elsif ( $self->Name eq 'Bookmarks' ) {
+        my $content = $self->Content;
+        my @ids;
+        for my $uid (@$content) {
+            if ( my $ticket = $importer->LookupObj($$uid) ) {
+                push @ids, $ticket->Id;
+            }
+        }
+        $content = { map { $_ => 1 } @ids };
+        $self->SetContent($content);
+    }
 }
 
 sub PostInflate {
@@ -1069,18 +1104,13 @@ sub Serialize {
         my $content = $self->_DeserializeContent($store{Content});
         for my $pane (values %{ $content || {} }) {
             for (@$pane) {
-                my $attr = RT::Attribute->new($self->CurrentUser);
-                $attr->LoadById($_);
-                $_ = \($attr->UID);
+                $_ = \( join '-', 'RT::Attribute', $RT::Organization, $_ );
             }
         }
         $store{Content} = $self->_SerializeContent($content);
     }
     elsif ( $store{Name} =~ /DefaultDashboard$/ ) {
-        my $content = $store{Content};
-        my $attr    = RT::Attribute->new( $self->CurrentUser );
-        $attr->LoadById($content);
-        $store{Content} = \$attr->UID;
+        $store{Content} = \( join '-', 'RT::Attribute', $RT::Organization, $store{Content} );
     }
     # encode saved searches and dashboards to be UIDs
     elsif ($store{Name} eq 'Dashboard') {
@@ -1088,9 +1118,7 @@ sub Serialize {
         for my $pane (values %{ $content->{Panes} || {} }) {
             for (@$pane) {
                 if ($_->{portlet_type} eq 'search' || $_->{portlet_type} eq 'dashboard') {
-                    my $attr = RT::Attribute->new($self->CurrentUser);
-                    $attr->LoadById($_->{id});
-                    $_->{uid} = \($attr->UID);
+                    $_->{uid} = \( join '-', 'RT::Attribute', $RT::Organization, $_->{id} );
                 }
                 # pass through everything else (e.g. component)
             }
@@ -1100,9 +1128,30 @@ sub Serialize {
     # encode subscriptions to have dashboard UID
     elsif ($store{Name} eq 'Subscription') {
         my $content = $self->_DeserializeContent($store{Content});
-        my $attr = RT::Attribute->new($self->CurrentUser);
-        $attr->LoadById($content->{DashboardId});
-        $content->{DashboardId} = \($attr->UID);
+        $content->{DashboardId} = \( join '-', 'RT::Attribute', $RT::Organization, $content->{DashboardId} );
+
+        # encode user/groups to be UIDs
+        for my $type (qw/Users Groups/) {
+            if ( $content->{Recipients}{$type} ) {
+                my $class = $type eq 'Users' ? 'RT::User' : 'RT::Group';
+                my @uids;
+                for my $id ( @{ $content->{Recipients}{$type} } ) {
+                    my $obj = $class->new( RT->SystemUser );
+                    $obj->Load($id);
+                    if ( $obj->Id ) {
+                        push @uids,
+                            \( join '-', $class, $class eq 'RT::User' ? $obj->Name : ( $RT::Organization, $obj->Id ) );
+                    }
+                }
+                $content->{Recipients}{$type} = \@uids;
+            }
+        }
+
+        $store{Content} = $self->_SerializeContent($content);
+    }
+    elsif ( $self->Name eq 'Bookmarks' ) {
+        my $content = $self->Content;
+        $content = [ map { \( join '-', 'RT::Ticket', $RT::Organization, $_ ) } keys %$content ];
         $store{Content} = $self->_SerializeContent($content);
     }
 
