@@ -271,6 +271,12 @@ sub BuildMainNav {
         );
     }
 
+    $tools->child( preview_searches =>
+        title       => loc('Search Modules'),
+        description => loc('Preview results of search modules'),
+        path        => '/Tools/PreviewSearches.html',
+    );
+
     if ( $current_user->HasRight( Right => 'ShowConfigTab', Object => RT->System ) )
     {
         _BuildAdminMenu( $request_path, $top, $widgets, $page, %args );
@@ -603,7 +609,6 @@ sub BuildMainNav {
 
         $has_query = 1 if ( $HTML::Mason::Commands::DECODED_ARGS->{'Query'} or $current_search->{'Query'} );
 
-        my %query_args;
         my %fallback_query_args = (
             SavedSearchId => ( $search_id eq 'new' || $search_id eq $chart_id ) ? undef : $search_id,
             SavedChartSearchId => $chart_id,
@@ -668,7 +673,7 @@ sub BuildMainNav {
         }
 
         my %short_query = ShortenSearchQuery(%final_query_args);
-        $args = '?' . QueryString(%short_query);
+        $args = '?' . QueryString(%short_query) if keys %short_query;
 
         my $current_search_menu;
         if (   $class eq 'RT::Tickets' && $request_path =~ m{^/Ticket}
@@ -726,7 +731,7 @@ sub BuildMainNav {
         }
 
         $current_search_menu->child( edit_search =>
-            title => loc('Edit Search'), sort_order => 1, path => "/Search/Build.html" . ( ($has_query) ? $args : '' ) );
+            title => loc('Edit Search'), sort_order => 1, path => "/Search/Build.html$args" );
         if ( $current_user->HasRight( Right => 'ShowSearchAdvanced', Object => RT->System ) ) {
             $current_search_menu->child( advanced => title => loc('Advanced'), path => "/Search/Edit.html$args" );
         }
@@ -871,7 +876,6 @@ sub BuildMainNav {
         my $has_query;
         $has_query = 1 if ( $HTML::Mason::Commands::DECODED_ARGS->{'Query'} or $current_search->{'Query'} );
 
-        my %query_args;
         my %fallback_query_args = (
             Class => 'RT::Assets',
             SavedSearchId => ( $search_id eq 'new' ) ? undef : $search_id,
@@ -1567,6 +1571,7 @@ sub _BuildAdminMenu {
 
             $page->child( basics => title => loc('Basics') => path => "/Admin/Scrips/Modify.html?id=" . $id . $from_query_param );
             $page->child( 'applies-to' => title => loc('Applies to'), path => "/Admin/Scrips/Objects.html?id=" . $id . $from_query_param );
+            $page->child( 'logging' => title => loc('Log Output'), path => "/Admin/Scrips/Logging.html?id=" . $id . $from_query_param );
         }
         elsif ( $request_path =~ m{^/Admin/Scrips/(index\.html)?$} ) {
             HTML::Mason::Commands::PageMenu->child( select => title => loc('Select') => path => "/Admin/Scrips/" );
