@@ -2,7 +2,7 @@
 #
 # COPYRIGHT:
 #
-# This software is Copyright (c) 1996-2015 Best Practical Solutions, LLC
+# This software is Copyright (c) 1996-2023 Best Practical Solutions, LLC
 #                                          <sales@bestpractical.com>
 #
 # (Except where explicitly superseded by other copyright notices)
@@ -50,15 +50,21 @@ package RT::ObjectCustomFieldValues;
 
 use strict;
 use warnings;
+use 5.010;
 
 use base 'RT::SearchBuilder';
 
 use RT::ObjectCustomFieldValue;
 
+# Set up the OCFV cache for faster comparison on add/update
+our $_OCFV_CACHE;
+ClearOCFVCache();
+
 sub Table { 'ObjectCustomFieldValues'}
 
 sub _Init {
     my $self = shift;
+    $self->{'with_disabled_column'} = 1;
 
   # By default, order by SortOrder
   $self->OrderByCols(
@@ -73,6 +79,15 @@ sub _Init {
     return ( $self->SUPER::_Init(@_) );
 }
 
+=head2 ClearOCFVCache
+
+Cleans out and reinitializes the OCFV cache
+
+=cut
+
+sub ClearOCFVCache {
+    $_OCFV_CACHE = {}
+}
 
 # {{{ sub LimitToCustomField
 
@@ -128,10 +143,30 @@ sub HasEntry {
     my $large_content = shift;
     return undef unless defined $value && length $value;
 
+    my $first = $self->First;
+    return undef unless $first;  # No entries to check
+
+    # Key should be the same for all values of the same ocfv
+    my $ocfv_key = $first->GetOCFVCacheKey;
+
+    # This cache relieves performance issues when adding large numbers of values
+    # to a CF since each add compares against the full list each time.
+
+    unless ( $_OCFV_CACHE->{$ocfv_key} ) {
+        # Load the cache with existing values
+        foreach my $item ( @{$self->ItemsArrayRef} ) {
+            push @{$_OCFV_CACHE->{$ocfv_key}}, {
+                'ObjectId'       => $item->Id,
+                'CustomFieldObj' => $item->CustomFieldObj,
+                'Content'        => $item->_Value('Content'),
+                'LargeContent'   => $item->LargeContent };
+        }
+    }
+
     my %canon_value;
-    #TODO: this could cache and optimize a fair bit.
-    foreach my $item ( @{$self->ItemsArrayRef} ) {
-        my $cf = $item->CustomFieldObj;
+    my $item_id;
+    foreach my $item ( @{$_OCFV_CACHE->{$ocfv_key}} ) {
+        my $cf = $item->{'CustomFieldObj'};
         my $args = $canon_value{ $cf->Type };
         if ( !$args ) {
             $args = { Content => $value, LargeContent => $large_content };
@@ -142,50 +177,54 @@ sub HasEntry {
 
         if ( $cf->Type eq 'Select' ) {
             # select is case insensitive
-            return $item if lc $item->Content eq lc $args->{Content};
+            $item_id = $item->{'ObjectId'} if lc $item->{'Content'} eq lc $args->{Content};
         }
         else {
-            if ( ($item->_Value('Content') || '') eq $args->{Content} ) {
-                if ( defined $item->LargeContent ) {
-                    return $item
+            if ( ($item->{'Content'} // '') eq $args->{Content} ) {
+                if ( defined $item->{'LargeContent'} ) {
+                    $item_id = $item->{'ObjectId'}
                       if defined $args->{LargeContent}
-                      && $item->LargeContent eq $args->{LargeContent};
+                      && $item->{'LargeContent'} eq $args->{LargeContent};
                 }
                 else {
-                    return $item unless defined $args->{LargeContent};
+                    $item_id = $item->{'ObjectId'} unless defined $args->{LargeContent};
                 }
-            } elsif ( $item->LargeContent && $args->{Content} ) {
-                return $item if ($item->LargeContent eq $args->{Content});
+            } elsif ( $item->{'LargeContent'} && $args->{Content} ) {
+                $item_id = $item->{'ObjectId'} if ($item->{'LargeContent'} eq $args->{Content});
             }
         }
+        last if $item_id;
+    }
+
+    if ( $item_id ) {
+        my $ocfv = RT::ObjectCustomFieldValue->new( $self->CurrentUser );
+        my ($ret, $msg) = $ocfv->Load($item_id);
+        RT::Logger->error("Unable to load object custom field value from id: $item_id $msg")
+            unless $ret;
+        return $ocfv;
+    }
+    else {
+        return undef;
     }
-    return undef;
 }
 
-sub _DoSearch {
-    my $self = shift;
-    
-    # unless we really want to find disabled rows,
-    # make sure we're only finding enabled ones.
-    unless ( $self->{'find_expired_rows'} ) {
-        $self->LimitToEnabled();
-    }
-    
-    return $self->SUPER::_DoSearch(@_);
-}
 
-sub _DoCount {
+sub AddRecord {
     my $self = shift;
-    
-    # unless we really want to find disabled rows,
-    # make sure we're only finding enabled ones.
-    unless ( $self->{'find_expired_rows'} ) {
-        $self->LimitToEnabled();
-    }
-    
-    return $self->SUPER::_DoCount(@_);
+    my ($record) = @_;
+
+    return unless $record->CurrentUserCanSee;
+    return $self->SUPER::AddRecord($record);
 }
 
 RT::Base->_ImportOverlays();
 
+# Clear the OCVF cache on exit to release connected RT::Ticket objects.
+#
+# Without this, there could be warnings generated like "Too late to safely run
+# transaction-batch scrips...". You can test this by commenting it out and running
+# some cf tests, e.g. perl -Ilib t/customfields/enter_one.t
+END { ClearOCFVCache(); }
+
+
 1;
