@@ -190,7 +190,7 @@ function checkboxToInput(target,checkbox,val){
 
     if(box.prop('checked')){
         if ( emails.indexOf(val) == -1 ) {
-            emails.unshift(val);
+            emails.push(val);
         }
     }
     else{
@@ -219,20 +219,37 @@ function checkboxesToInput(target,checkboxes) {
         return email.match(/\S/) ? true : false;
     });
 
+    var selectize = tar[0].selectize;
+    var added = [];
+    var removed = [];
+
     jQuery(checkboxes).each(function(index, checkbox) {
         var val = jQuery(checkbox).attr('data-address');
         if(jQuery(checkbox).prop('checked')){
             if ( emails.indexOf(val) == -1 ) {
-                emails.unshift(val);
+                emails.push(val);
+                added.push(val);
             }
         }
         else{
             emails = jQuery.grep(emails, function(email) {
                 return email != val;
             });
+            removed.push(val);
         }
     });
 
+    if ( selectize ) {
+
+        // Add new items in one call to avoid triggering syncOneTimeCheckboxes
+        // multiple times during the update as it could wrongly sync the
+        // incomplete input values back to checkboxes.
+        selectize.addItems(added, true);
+        for ( const item of removed ) {
+            selectize.removeItem(item, true);
+        }
+    }
+
     jQuery('#UpdateIgnoreAddressCheckboxes').val(true);
     tar.val(emails.join(', ')).change();
 }
@@ -354,11 +371,7 @@ function textToHTML(value) {
 
 CKEDITOR_BASEPATH=RT.Config.WebPath + "/static/RichText/";
 function ReplaceAllTextareas() {
-    var sAgent = navigator.userAgent.toLowerCase();
-    if (!CKEDITOR.env.isCompatible ||
-        sAgent.indexOf('iphone') != -1 ||
-        sAgent.indexOf('ipad') != -1 ||
-        sAgent.indexOf('android') != -1 )
+    if (!CKEDITOR.env.isCompatible)
         return false;
 
     // replace all content and signature message boxes
@@ -1144,11 +1157,6 @@ function filterSearchResults () {
 /* inline edit */
 jQuery(function () {
     var inlineEditEnabled = true;
-    var disableInlineEdit = function () {
-        inlineEditEnabled = false;
-        jQuery('.editable').removeClass('editing').removeClass('loading');
-        jQuery('table.inline-edit').removeClass('inline-edit');
-    };
 
     var escapeKeyHandler = null;
 
@@ -1245,9 +1253,24 @@ jQuery(function () {
 
         var renderError = function (error) {
             jQuery.jGrowl(error, { sticky: true, themeState: 'none' });
-            cell.addClass('error text-danger').html(loc_key('error'));
+            cell.removeClass('loading');
+            tbody.removeClass('refreshing');
+            editor.find(':input').removeAttr('disabled');
+            var errorMessage = jQuery('<div>'+loc_key('error')+'</div>')
+                .addClass('error text-danger').hide();
+            var fadeTime = 250;
+            cell.find('div.value').fadeOut(fadeTime,function () {
+                cell.append(errorMessage);
+                errorMessage.fadeIn(fadeTime, function () {
+                    setTimeout(function () {
+                        errorMessage.fadeOut(fadeTime, function () {
+                            errorMessage.remove();
+                            cell.find('div.value').fadeIn(fadeTime);
+                        });
+                    }, 2000);
+                });
+            });
             jQuery(document).off('keyup', escapeKeyHandler);
-            disableInlineEdit();
         };
         jQuery.ajax({
             url     : editor.attr('action'),
